from django.contrib import admin
from .models import Entry

# Register your models here.
@admin.register(Entry)
class EntryAdmin(admin.ModelAdmin):
    list_display = ('key', 'value', 'type', 'enabled', 'comment', )
    list_filter = ('type', 'enabled')



# Generated by Django 2.2.6 on 2020-04-04 07:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('registry', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='type',
            field=models.CharField(blank=True, choices=[('str', 'string'), ('int', 'integer'), ('float', 'float'), ('bool', 'bool')], default='str', max_length=8, verbose_name='Type'),
        ),
    ]



from .models import Entry

class Meta(type):
    def __getitem__(self, key):
        try:
            entry = Entry.objects.get(key=key, enabled=True)
        except Entry.DoesNotExist as e:
            raise KeyError(key) from e

        if entry.type == Entry.Types.STRING:
            return entry.value
        elif entry.type == Entry.Types.INTEGER:
            return int(entry.value)
        elif entry.type == Entry.Types.FLOAT:
            return float(entry.value)
        elif entry.type == Entry.Types.BOOLEAN:
            return True if entry.value == 'true' else False

        raise TypeError

    def __setitem__(self, key, value):
        if value is None:
            raise ValueError('Cannot set registry value to None')

        try:
            entry = Entry.objects.get(key=key)
        except Entry.DoesNotExist:
            entry = Entry(key=key)

        entry.value = value

        if type(value) == int:
            entry.type = Entry.Types.INTEGER
        elif type(value) == float:
            entry.type = Entry.Types.FLOAT
        elif type(value) == bool:
            entry.type = Entry.Types.BOOLEAN
            entry.value = str(value).lower()
        else:
            entry.type = Entry.Types.STRING

        entry.enabled = True
        entry.save()

    def __delitem__(self, key):
        try:
            entry = Entry.objects.get(key=key)
        except Entry.DoesNotExist as e:
            raise KeyError(key) from e

        entry.delete()

    def __iter__(self):
        return iter(Entry.objects.filter(enabled=True))

    def __contains__(self, key):
        return Entry.objects.filter(key=key).count() > 0


class reg(metaclass=Meta):
    @classmethod
    def get(cls, key, default=None):
        try:
            return cls[key]
        except KeyError:
            return default



import os
import re

from setuptools import find_packages, setup


BASE_DIR = os.path.realpath(os.path.dirname(__file__))

def read(*parts):
    with open(os.path.join(BASE_DIR, *parts), 'r', encoding='utf-8') as f:
        return f.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r'^__version__ = ["\']([^"\']*)["\']', version_file, re.M)

    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name='dj_registry',
    version=find_version('registry', '__init__.py'),
    url='https://github.com/yychen/dj-registry',
    license='MIT',
    description='A simple, easy access key-value registry for Django.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='Tom Chiung-ting Chen',
    author_email='ctchen@gmail.com',
    packages=find_packages(exclude=['tests*']),
    python_requires=">=3",
    zip_safe=False,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


# Create your models here.
class Entry(models.Model):
    class Types:
        STRING = 'str'
        INTEGER = 'int'
        FLOAT = 'float'
        BOOLEAN = 'bool'

        choices = (
            (STRING, 'string'),
            (INTEGER, 'integer'),
            (FLOAT, 'float'),
            (BOOLEAN, 'bool'),
        )

    key = models.CharField('Key', max_length=64, db_index=True)
    value = models.CharField('Value', max_length=512, blank=True)
    type = models.CharField('Type', max_length=8, choices=Types.choices, blank=True, default=Types.STRING)
    enabled = models.BooleanField('Enabled', default=True)
    comment = models.TextField('Comment', blank=True)

    def __str__(self):
        return self.key

    def clean(self):
        try:
            if self.type == self.Types.INTEGER:
                int(self.value)
            elif self.type == self.Types.FLOAT:
                float(self.value)
            elif self.type == self.Types.BOOLEAN:
                if self.value.lower() in ['true', 'yes', '1']:
                    self.value = 'true'

                if self.value.lower() in ['false', 'no', '0']:
                    self.value = 'false'

                if self.value != 'true' and self.value != 'false':
                    raise ValidationError({'value': _('Accepted boolean values: true, false, yes, no, 0, 1')})

        except ValueError:
            raise ValidationError({'value': _('Invalid value of the specified type.')})


    class Meta:
        verbose_name_plural = 'entries'



# Generated by Django 2.2.6 on 2019-10-16 08:33

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(db_index=True, max_length=64, verbose_name='Key')),
                ('value', models.CharField(blank=True, max_length=512, verbose_name='Value')),
                ('type', models.CharField(blank=True, choices=[('str', 'string'), ('int', 'integer'), ('float', 'float')], default='str', max_length=8, verbose_name='Type')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled')),
                ('comment', models.TextField(blank=True, verbose_name='Comment')),
            ],
            options={
                'verbose_name_plural': 'entries',
            },
        ),
    ]
